#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define TCPV40HDRS

#include <sys/socket.h>
#include <netinet/in.h>

#pragma	library("SO32DLL")
#pragma	library("TCP32DLL")

void dumpbuf(unsigned char * buf, int len)
{
   int i;
   if (len <= 0) return;
   for (i = 0; i < len; i++) printf("%02X ", buf[i]);
   printf("\n");
}

void usage(void)
{
  printf("test <mapper ip> <mapper port> <login> <password>\n");
  exit(1);
}

void main(int argc, char ** argv)
{
   struct sockaddr_in saddr;
   int sock, rc;
   unsigned char buf[1024];

   if (argc < 4) usage();

   sock_init();

   sock = socket(PF_INET, SOCK_STREAM, 0);
   printf("socket %d\n", sock);
   if (sock < 0) exit(1);

   saddr.sin_addr.s_addr = inet_addr(argv[1]);
   saddr.sin_port = htons(atoi(argv[2])) & 0xFFFF;
   saddr.sin_family = AF_INET;

   rc = connect(sock, (struct sockaddr *)&(saddr), sizeof(saddr));
   printf("connect %d\n", rc);
   if (rc) exit(1);

   // sending auth request of format : 
   // --------------------------------------------------------------------------------------
   // | header (0x01) | number of supported methods (1 byte) | methods (1 method - 1 byte) |
   // --------------------------------------------------------------------------------------
   buf[0] = 0x01; // protocol version for TCPMAP auth
   buf[1] = 0x01; // number of supported auth methods
   buf[2] = 0x02; // auth method - password auth
   rc = send(sock, buf, 3, 0);
   printf("send %d\n", rc);
   if (rc != 3) exit(1);

   // receiving response
   rc = recv(sock, buf, 1024, 0);
   printf("recv %d\n", rc);
   printf("Choosed auth method response: \n");
   dumpbuf(buf, rc);
   if (rc < 2) exit(1);
   // response has format :
   // -------------------------------------------
   // | header (0x01) | choosed method (1 byte) |
   // -------------------------------------------
   // the choosed auth method shoud be - 0x02 for password auth
   if (buf[0] != 0x01 || buf[1] != 0x02) exit(1);

   // constructing auth packet : 
   // ------------------------------------------------------------------------------------
   // | header (0x01) | login length (1 byte) | login | passw length (1 byte) | password |
   // ------------------------------------------------------------------------------------
   // password auth version - according to socks5 rfc
   buf[0] = 0x01; 
   // the length of login
   buf[1] = strlen(argv[3]) & 0xFF;
   // login itself
   memcpy(buf + 2, argv[3], buf[1]);
   // the length of password
   buf[2+buf[1]] = strlen(argv[4]) & 0xFF;
   // password itself
   memcpy(buf + 3 + buf[1], argv[4], buf[2+buf[1]]);

   printf("Auth packet: \n");
   dumpbuf(buf, 3 + buf[1] + buf[2+buf[1]]);
   rc = send(sock, buf, 3 + buf[1] + buf[2+buf[1]], 0);
   printf("send %d\n", rc);
   if (rc != 3 + buf[1] + buf[2+buf[1]]) exit(1);

   // receiving response
   rc = recv(sock, buf, 1024, 0);
   printf("recv %d\n", rc);
   printf("Auth response: \n");
   dumpbuf(buf, rc);
   if (rc < 2) exit(1);
   // response has format :
   // ----------------------------------------
   // | header (0x01) | auth result (1 byte) |
   // ----------------------------------------
   // auth result = 0x00 - auth passed, otherwise auth failed
   if (buf[0] != 0x01 || buf[1] != 0x00) exit(1);
   printf("Auth passed\n");

   // now you are connected to the mapper and can do your communication

   exit(0);
}
